<?php

namespace Drp\JsonApi\Exceptions;

use Illuminate\Http\Response;

class JsonApiException extends \Exception
{
    public const INVALID_REQUEST = 7;

    /**
     * @var integer
     */
    private $httpStatusCode;

    /**
     * @var string
     */
    private $errorType;

    /**
     * @var null|string
     */
    private $hint;

    /**
     * Throw a new exception.
     *
     * @param string $message Error message
     * @param integer $code Error code
     * @param string $errorType Error type
     * @param integer $httpStatusCode HTTP status code to send (default = 400)
     * @param null|string $hint A helper hint
     */
    public function __construct(
        string $message,
        int $code,
        string $errorType,
        int $httpStatusCode = 400,
        ?string $hint = null
    ) {
        parent::__construct($message, $code);
        $this->httpStatusCode = $httpStatusCode;
        $this->errorType = $errorType;
        $this->hint = $hint;
    }

    /**
     * Request contains invalid json payload.
     *
     * @return static
     */
    public static function invalidRequest(): self
    {
        return new static(
            'Invalid Json Specified',
            self::INVALID_REQUEST,
            'invalid_request',
            Response::HTTP_BAD_REQUEST,
            'Please check your request and try again.'
        );
    }

    /**
     * Returns the HTTP status code to send when the exceptions is output.
     */
    public function getHttpStatusCode() : int
    {
        return $this->httpStatusCode;
    }

    public function getHint(): ?string
    {
        return $this->hint;
    }
}
