<?php

namespace Drp\JsonApi\Http\Middleware;

use Closure;
use Drp\JsonApi\Http\Concerns\RespondsWithJsonApi;
use Drp\JsonApi\Http\Transformers\TokenTransformer;
use Illuminate\Http\Request;
use League\Fractal\Serializer\JsonApiSerializer;
use stdClass;
use Symfony\Component\HttpFoundation\Response;

class ConvertToJsonApiForPassport
{
    use RespondsWithJsonApi;

    public function handle(Request $request, Closure $next): Response
    {
        $response = $next($request);
        $content = json_decode($response->getContent());
        if ($response->isSuccessful() === false) {
            return $this->respondWithError(
                $response->getStatusCode(),
                Response::$statusTexts[$response->getStatusCode()],
                $this->getErrorResponseDetail($content)
            );
        }

        if ($content === null) {
            return $response;
        }

        return $response->setContent($this->handleSuccess($content));
    }

    private function handleSuccess(?stdClass $responseContent): array
    {
        return fractal()
            ->serializeWith(new JsonApiSerializer)
            ->item($responseContent, new TokenTransformer, 'token')
            ->toArray();
    }


    private function getErrorResponseDetail(?stdClass $content): string
    {
        if ($content === null) {
            return 'Check your error logs for more information.';
        }

        return (property_exists($content, 'errors') ? $content->errors->detail : $content->message);
    }
}
