<?php

namespace Drp\JsonApi\Http\Middleware;

use Closure;
use Drp\JsonApi\Exceptions\JsonApiException;
use Drp\JsonApi\Http\Concerns\RespondsWithJsonApi;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;

class ParseJsonApiForPassport
{
    use RespondsWithJsonApi;

    public function handle(Request $request, Closure $next): Response
    {
        try {
            $request = $this->fromJsonApi($request);
        } catch (JsonApiException $exception) {
            return $this->respondWithError(
                $exception->getHttpStatusCode(),
                $exception->getMessage(),
                $exception->getHint()
            );
        }

        return $next($request);
    }

    /**
     * Convert the JSON API structure to form params so Passport can read them.
     *
     * @throws \Drp\JsonApi\Exceptions\JsonApiException
     */
    private function fromJsonApi(Request $request): Request
    {
        $decodedRequest = json_decode($request->getContent(), true);

        if (json_last_error()) {
            throw JsonApiException::invalidRequest();
        }

        $attributes = array_get($decodedRequest, 'data.attributes', []);

        if (\is_array($attributes)) {
            foreach ($attributes as $key => $value) {
                $request->request->set(snake_case($key), $value);
            }
        }

        return $request;
    }
}
