<?php

namespace Drp\JsonApi\Http\Middleware;

use Closure;
use Drp\JsonApi\Http\Concerns\RespondsWithJsonApi;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Symfony\Component\HttpFoundation\Response as SymfonyResponse;

class ValidateJsonApiHeader
{
    use RespondsWithJsonApi;

    public function handle(Request $request, Closure $next): SymfonyResponse
    {
        if ($this->shouldCheckContentType($request) && $this->hasInvalidContentType($request)) {
            return $this->respondWithError(
                Response::HTTP_NOT_ACCEPTABLE,
                'Not Acceptable',
                "Invalid Content-Type, 'application/vnd.api+json' expected"
            );
        }

        return $next($request);
    }

    private function shouldCheckContentType(Request $request) : bool
    {
        return \in_array($request->method(), ['POST', 'PATCH', 'PUT', 'DELETE']);
    }

    private function hasInvalidContentType(Request $request): bool
    {
        return Request::matchesType($request->header('content-type'), 'application/vnd.api+json') === false &&
            $request->getContent() !== '';
    }
}
